# Generated by Django 3.1.10 on 2021-05-25 17:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("authentication", "0017_remove_fleetup_permission"),
        ("killtracker", "0002_fix_webhook_notes_field"),
    ]

    operations = [
        migrations.AddField(
            model_name="tracker",
            name="exclude_attacker_states",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="exclude killmails with characters belonging to users with these Auth states",
                related_name="_tracker_exclude_attacker_states_+",
                to="authentication.State",
            ),
        ),
        migrations.AddField(
            model_name="tracker",
            name="require_attacker_states",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="only include killmails with characters belonging to users with these Auth states",
                related_name="_tracker_require_attacker_states_+",
                to="authentication.State",
            ),
        ),
        migrations.AddField(
            model_name="tracker",
            name="require_victim_states",
            field=models.ManyToManyField(
                blank=True,
                default=None,
                help_text="only include killmails where the victim characters belong to users with these Auth states",
                related_name="_tracker_require_victim_states_+",
                to="authentication.State",
            ),
        ),
    ]
